/**
 * Copyright (C) 2015 Justin James.
 *
 * This license must be preserved.
 * Any applications, libraries, or code which make any use of any
 * component of this program must not be commercial, unless explicit
 * permission is granted from the original author. The use of this
 * program for non-profit purposes is permitted.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * In the event that this license restricts you from making desired use of this program, contact the original author.
 * Written by Justin James <justin.aj@hotmail.com>
 */

class eX_RconConnection extends Rx_RconConnection;

var byte cmd;

function commandResponse(string response)
{
	SendText("xr" $ Chr(cmd) $ response);
}

function commandError(string response)
{
	SendText("xe" $ Chr(cmd) $ response);
}

function int getAveragePing()
{
	local PlayerReplicationInfo lPRI;
	local int ping;
	local int count;
	foreach WorldInfo.Game.GameReplicationInfo.PRIArray(lPRI)
	{
		if (lPRI.PlayerID != 0 && lPRI.bBot == false)
		{
			ping += lPRI.Ping;
			count++;
		}
	}
	if (count == 0)
		return 0;
	return ping/(float(count)/4);
}

function PlayerReplicationInfo getPlayerByID(int id)
{
	local PlayerReplicationInfo lPRI;
	foreach WorldInfo.Game.GameReplicationInfo.PRIArray(lPRI)
		if (lPRI.PlayerID == id)
			return lPRI;
	return None;
}

function PingScoreCreditsList()
{
	local PlayerReplicationInfo lPRI;
	local byte oldCmd;
	oldCmd = cmd;
	cmd = 2;
	foreach WorldInfo.Game.GameReplicationInfo.PRIArray(lPRI)
		if (lPRI.PlayerID != 0 && Rx_PRI(lPRI) != None)
			commandResponse(class'Rx_PRI'.static.LogNameOf(lPRI) $ Chr(160) $ lPRI.Ping * 4 $ Chr(160) $ Rx_PRI(lPRI).getRenScore() $ Chr(160) $ Rx_PRI(lPRI).GetCredits());
	cmd = oldCmd;
}

event ReceivedLine(string Text)
{
	local PlayerReplicationInfo pri;
	local Rx_PRI RxPRI;
	local string data;
	local array<string> dataParts;
	local float credits;
	local float time;
	local byte options;
	local Controller cnt;

	if (bAuthd && Len(Text) > 2 && Left(Text,2) == "_x")
	{
		cmd = Asc(Mid(Text,2,1));
		switch (cmd)
		{
			case 1:	// Client List
				foreach WorldInfo.Game.GameReplicationInfo.PRIArray(pri)
				{
					RxPRI = Rx_PRI(pri);
					if (RxPRI != None && RxPRI.PlayerID != 0)
						commandResponse(class'Rx_PRI'.static.LogNameOf(RxPRI) $ Chr(160) $ RxPRI.SavedNetworkAddress $ Chr(160) $ (Class'GameEngine'.static.GetOnlineSubsystem().UniqueNetIdToString(RxPRI.UniqueId)) $ Chr(160) $ RxPRI.StartTime $ Chr(160) $ RxPRI.Ping * 4 $ Chr(160) $ RxPRI.Kills $ Chr(160) $ RxPRI.Deaths $ Chr(160) $ RxPRI.getRenScore() $ Chr(160) $ RxPRI.GetCredits() $ Chr(160) $ RxPRI.CharClassInfo.name);
				}
				break;
			case 2: // Ping & Score & Credits List
				PingScoreCreditsList();
				break;
			case 3: // Echo
				commandResponse(Mid(Text,3));
				break;
			case 4: // Add credits (Default: 1 credit)
				data = Mid(Text,3);
				if (Len(data) != 0)
				{
					ParseStringIntoArray(data, dataParts, Chr(160), false);
					if (dataParts.Length == 1)
						credits = 1.0;
					else
						credits = float(dataParts[1]);
					RxPRI = Rx_PRI(getPlayerByID(int(dataParts[0])));
					if (RxPRI != None)
					{
						RxPRI.AddCredits(credits);
						commandResponse(class'Rx_PRI'.static.LogNameOf(RxPRI) $ Chr(160) $ string(int(RxPRI.GetCredits())));
					}
					else
						commandError("PNF");
				}

				break;
			case 5: // Ping (Average or specific)
				data = Mid(Text,3);
				if (Len(data) == 0) // Average ping
					commandResponse(string(getAveragePing()));
				else // Player-specific ping
				{
					pri = getPlayerByID(int(data));
					if (pri != None)
						commandResponse(class'Rx_PRI'.static.LogNameOf(pri) $ Chr(160) $ pri.Ping);
					else
						commandResponse("PNF");
				}
				break;
			case 6: // Subscribe to Ping & Score & Credits.
				data = Mid(Text,3);
				if (Len(data) == 0)
					time = 1.0f;
				else
					time = float(data);
				SetTimer(time, true, 'PingScoreCreditsList');
				commandResponse(string(time));
				break;
			case 7: // Force team change - Player ID [| Options[| Team]]]
					// Options: 00<Reset Kills><Reset Deaths><Force-Zero><Reset Score=0><Reset Credits=1><Respawn=1>
				data = Mid(Text,3);
				if (Len(data) != 0)
				{
					ParseStringIntoArray(data, dataParts, Chr(160), false);
					RxPRI = Rx_PRI(getPlayerByID(int(dataParts[0])));
					if (RxPRI != None)
					{
						cnt = Controller(RxPRI.Owner);

						if (dataParts.Length == 1)
							options = 0x03;
						else
							options = byte(Asc(dataParts[1]));

						// Pre-swap option processing
						if ((options & 0x08) != 0)
							options = 0;
						else
						{
							if ((options & 0x02) != 0)
								credits = Rx_Game(WorldInfo.Game).InitialCredits;
							else
								credits = RxPRI.GetCredits();

							if ((options & 0x04) != 0)
								RxPRI.AddScoreToPlayerAndTeam(RxPRI.GetRenScore(), false);
							if ((options & 0x10) != 0)
								RxPRI.Deaths = 0;
							if ((options & 0x20) != 0)
								RxPRI.Kills = 0;
						}

						// Swap
						if (dataParts.Length < 3)
						{
							if (RxPRI.GetTeamNum() == 0)
								RxPRI.SetPlayerTeam(Rx_Game(WorldInfo.Game).Teams[1]);
							else
								RxPRI.SetPlayerTeam(Rx_Game(WorldInfo.Game).Teams[0]);
						}
						else
							RxPRI.SetPlayerTeam(Rx_Game(WorldInfo.Game).Teams[int(dataParts[2])]);
						commandResponse(class'Rx_PRI'.static.LogNameOf(RxPRI) $ Chr(160) $ options);

						//Post-swap option processing
						if ((options & 0x01) != 0)
						{
							if (cnt != None)
							{
								if (cnt.Pawn != None)
									cnt.Pawn.Destroy();
								cnt.ServerRestartPlayer();
							}
						}
						RxPRI.SetCredits(credits);
					}
					else
						commandError("PNF");
				}
				break;
			case 10: // New-line - DO NOT USE
			case 13: // Carriage-Return - NO NOT USE
				break;
			default:
				break;
		}
		//data = Mid(Text,3);
		return;
	}
	Super.ReceivedLine(Text);
}